@@ -19,7 +19,7 @@ __all__ = ['Registry', 'SettingRegistry', 'ExporterRegistry',
            'DataExporterRegistry', 'data_exporter', 'layer_action',
            'SubsetMaskExporterRegistry', 'SubsetMaskImporterRegistry',
            'StartupActionRegistry', 'startup_action', 'QtFixedLayoutTabRegistry',
-           'qt_fixed_layout_tab']
+           'qt_fixed_layout_tab', 'KeyboardShortcut', 'keyboard_shortcut']
 
 
 CFG_DIR = os.path.join(os.path.expanduser('~'), '.glue')
@@ -180,11 +180,17 @@ class SettingRegistry(DictRegistry):
             value = self._members.get(key, self._defaults[key])
             yield key, value, self._validators[key]
 
+    def __str__(self):
+        s = ""
+        for name, value, validator in self:
+            s += "{0}: {1}\n".format(name, value)
+        return s
+
     def reset_defaults(self):
         self._members.clear()
 
     def is_default(self, setting):
-        return setting in self._defaults and not setting in self._members
+        return setting in self._defaults and setting not in self._members
 
 
 class QGlueParserRegistry(Registry):
@@ -361,6 +367,10 @@ class ColormapRegistry(Registry):
         import matplotlib.cm as cm
         members = []
         members.append(['Gray', cm.gray])
+        members.append(['Viridis', cm.viridis])
+        members.append(['Plasma', cm.plasma])
+        members.append(['Inferno', cm.inferno])
+        members.append(['Magma', cm.magma])
         members.append(['Purple-Blue', cm.PuBu])
         members.append(['Yellow-Green-Blue', cm.YlGnBu])
         members.append(['Yellow-Orange-Red', cm.YlOrRd])
@@ -379,6 +389,18 @@ class ColormapRegistry(Registry):
         """
         self.members.append([label, cmap])
 
+    def __getitem__(self, cmap_name):
+        for name, cmap in self.members:
+            if name == cmap_name:
+                return cmap
+        raise KeyError(cmap_name)
+
+    def name_from_cmap(self, cmap_desired):
+        for name, cmap in self.members:
+            if cmap is cmap_desired:
+                return name
+        raise ValueError("Could not find name for colormap")
+
 
 class DataFactoryRegistry(Registry):
 
@@ -601,12 +623,12 @@ class LayerActionRegistry(Registry):
         # Backward-compatibility
         if callback is not None:
             self.add(self.item(label, tooltip, callback, icon, True,
-                     False, False, True))
+                               False, False, True))
             return True
 
         def adder(func):
             self.add(self.item(label, tooltip, func, icon, single,
-                     data, subset_group, subset))
+                               data, subset_group, subset))
             return func
         return adder
 
@@ -670,6 +692,52 @@ class BooleanSetting(object):
         return self.state
 
 
+class KeyboardShortcut(DictRegistry):
+    """
+    Stores keyboard shortcuts.
+    The members property is a dictionary within a dictionary of keyboard
+    shortcuts, which is represented as (viewer,(keybind,function)). The
+    ``function`` should take one item, which is a reference to the session.
+    """
+
+    def add(self, valid_viewers, keybind, function):
+        """
+        Add a new keyboard shortcut
+
+        Parameters
+        ----------
+        arg1: list
+            list of viewers where event can be fired
+        arg2: Qt.Key
+            type of key event
+        arg3: function()
+            function to be run that corresponds with key
+        """
+        if valid_viewers:
+            for viewer in valid_viewers:
+                if viewer in self.members:
+                    if keybind in self.members[viewer]:
+                        raise ValueError("Keyboard shortcut '{0}' already registered in {1}".format(keybind, viewer))
+                    else:
+                        self.members[viewer][keybind] = function
+                else:
+                    self.members[viewer] = {keybind: function}
+        else:
+            if None in self.members:
+                if keybind in self.members[None]:
+                    raise ValueError("Keyboard shortcut '{0}' already registered in {1}".format(keybind, None))
+                else:
+                    self.members[None][keybind] = function
+            else:
+                self.members[None] = {keybind: function}
+
+    def __call__(self, keybind, valid_viewers):
+        def adder(func):
+            self.add(valid_viewers, keybind, func)
+            return func
+        return adder
+
+
 qt_client = QtClientRegistry()
 qt_fixed_layout_tab = QtFixedLayoutTabRegistry()
 viewer_tool = ViewerToolRegistry()
@@ -685,6 +753,7 @@ menubar_plugin = MenubarPluginRegistry()
 preference_panes = PreferencePanesRegistry()
 qglue_parser = QGlueParserRegistry()
 startup_action = StartupActionRegistry()
+keyboard_shortcut = KeyboardShortcut()
 
 # watch loaded data files for changes?
 auto_refresh = BooleanSetting(False)
@@ -748,7 +817,7 @@ def _default_search_order():
     return search_order[::-1]
 
 
-###### Now define global settings ######
+# ##### Now define global settings ######
 
 GRAY = '#7F7F7F'
 BLUE = "#1F78B4"
@@ -771,3 +840,5 @@ settings.add('DATA_ALPHA', 0.8, validator=float)
 settings.add('BACKGROUND_COLOR', '#FFFFFF')
 settings.add('FOREGROUND_COLOR', '#000000')
 settings.add('SHOW_LARGE_DATA_WARNING', True, validator=bool)
+settings.add('SHOW_INFO_PROFILE_OPEN', True, validator=bool)
+settings.add('SHOW_WARN_PROFILE_DUPLICATE', True, validator=bool)
