@@ -72,8 +72,7 @@ class Data(object):
 
         Extra array-like keywords are extracted into components
         """
-        # Coordinate conversion object
-        self.coords = coords or Coordinates()
+
         self._shape = ()
 
         # Components
@@ -83,6 +82,9 @@ class Data(object):
         self._pixel_component_ids = ComponentIDList()
         self._world_component_ids = ComponentIDList()
 
+        # Coordinate conversion object
+        self.coords = coords or Coordinates()
+
         self.id = ComponentIDDict(self)
 
         # Metadata
@@ -113,6 +115,20 @@ class Data(object):
         # uniquely identify them.
         self.uuid = str(uuid.uuid4())
 
+    @property
+    def coords(self):
+        """
+        The coordinates object for the data.
+        """
+        return self._coords
+
+    @coords.setter
+    def coords(self, value):
+        if (hasattr(self, '_coords') and self._coords != value) or not hasattr(self, '_coords'):
+            self._coords = value
+            if len(self.components) > 0:
+                self._update_world_components(self.ndim)
+
     @property
     def subsets(self):
         """
@@ -143,8 +159,11 @@ class Data(object):
     def label(self, value):
         """ Set the label to value
         """
-        self._label = value
-        self.broadcast(attribute='label')
+        if getattr(self, '_label', None) != value:
+            self._label = value
+            self.broadcast(attribute='label')
+        elif value is None:
+            self._label = value
 
     @property
     def size(self):
@@ -545,7 +564,10 @@ class Data(object):
         return dc
 
     def _create_pixel_and_world_components(self, ndim):
+        self._update_pixel_components(ndim)
+        self._update_world_components(ndim)
 
+    def _update_pixel_components(self, ndim):
         for i in range(ndim):
             comp = CoordinateComponent(self, i)
             label = pixel_label(i, ndim)
@@ -553,6 +575,10 @@ class Data(object):
             self.add_component(comp, cid)
             self._pixel_component_ids.append(cid)
 
+    def _update_world_components(self, ndim):
+        for cid in self._world_component_ids[:]:
+            self.remove_component(cid)
+            self._world_component_ids.remove(cid)
         if self.coords:
             for i in range(ndim):
                 comp = CoordinateComponent(self, i, world=True)
